@@ -152,7 +152,7 @@ class ScatterChart:
                            rescale_y=None,
                            original_x=None,
                            original_y=None):
-        '''
+        r'''
         Inject custom x and y coordinates for each term into chart.
 
         Parameters
@@ -217,6 +217,7 @@ class ScatterChart:
                 neutral_categories=None,
                 extra_categories=None,
                 background_scorer=None,
+                use_offsets=False,
                 **kwargs):
         '''
 
@@ -306,7 +307,7 @@ class ScatterChart:
             df[category_column_name]
         )
         df['color_scores'] = scores
-        if self.scatterchartdata.terms_to_include is None:
+        if self.scatterchartdata.terms_to_include is None and self.scatterchartdata.dont_filter is False:
             df = self._filter_bigrams_by_minimum_not_category_term_freq(
                 category_column_name, not_categories, df)
             df = filter_bigrams_by_pmis(
@@ -392,6 +393,10 @@ class ScatterChart:
             for term_obj in j['data']:
                 if term_obj['term'] in self.hidden_terms:
                     term_obj['display'] = False
+
+        if use_offsets:
+            j['offsets'] = self.term_doc_matrix.get_offsets()
+
         return j
 
     def _add_x_and_y_coords_to_term_df_if_injected(self, df):
@@ -433,14 +438,14 @@ class ScatterChart:
     def _add_term_freq_to_json_df(self, json_df, term_freq_df, category):
         json_df['cat25k'] = (((term_freq_df[category + ' freq'] * 1.
                                / term_freq_df[category + ' freq'].sum()) * 25000).fillna(0)
-                             .apply(np.round).astype(np.int))
+                             .apply(np.round).astype(int))
         json_df['ncat25k'] = (((term_freq_df['not cat freq'] * 1.
                                 / term_freq_df['not cat freq'].sum()) * 25000).fillna(0)
-                              .apply(np.round).astype(np.int))
+                              .apply(np.round).astype(int))
         if 'neut cat freq' in term_freq_df:
             json_df['neut25k'] = (((term_freq_df['neut cat freq'] * 1.
                                     / term_freq_df['neut cat freq'].sum()) * 25000).fillna(0)
-                                  .apply(np.round).astype(np.int))
+                                  .apply(np.round).astype(int))
             json_df['neut'] = term_freq_df['neut cat freq']
         else:
             json_df['neut25k'] = 0
@@ -448,7 +453,7 @@ class ScatterChart:
         if 'extra cat freq' in term_freq_df:
             json_df['extra25k'] = (((term_freq_df['extra cat freq'] * 1.
                                      / term_freq_df['extra cat freq'].sum()) * 25000).fillna(0)
-                                   .apply(np.round).astype(np.int))
+                                   .apply(np.round).astype(int))
             json_df['extra'] = term_freq_df['extra cat freq']
         else:
             json_df['extra25k'] = 0
@@ -504,7 +509,7 @@ class ScatterChart:
             df[category_column_name]
         )
         df['color_scores'] = scores
-        if self.scatterchartdata.terms_to_include is None:
+        if self.scatterchartdata.terms_to_include is None and self.scatterchartdata.dont_filter is False:
             df = self._filter_bigrams_by_minimum_not_category_term_freq(
                 category_column_name, other_categories, df)
             df = filter_bigrams_by_pmis(
@@ -525,7 +530,7 @@ class ScatterChart:
         return df
 
     def _filter_bigrams_by_minimum_not_category_term_freq(self, category_column_name, other_categories, df):
-        if self.scatterchartdata.terms_to_include is None:
+        if self.scatterchartdata.terms_to_include is None and self.scatterchartdata.dont_filter is False:
             return df[(df[category_column_name] > 0)
                       | (df[[c + ' freq' for c in other_categories]].sum(axis=1)
                          >= self.scatterchartdata.minimum_not_category_term_frequency)]
@@ -533,11 +538,12 @@ class ScatterChart:
             return df
 
     def _filter_by_minimum_term_frequency(self, all_categories, df):
-        if self.scatterchartdata.terms_to_include is None:
-            return df[df[[c + ' freq' for c in all_categories]].sum(axis=1)
-                      > self.scatterchartdata.minimum_term_frequency]
-        else:
-            return df
+        if self.scatterchartdata.terms_to_include is None and self.scatterchartdata.dont_filter is False:
+            df = df[
+                lambda df: (df[[c + ' freq' for c in all_categories]].sum(axis=1)
+                            > self.scatterchartdata.minimum_term_frequency)
+            ]
+        return df
 
     def _limit_max_terms(self, category, df):
         df['score'] = self._term_importance_ranks(category, df)
@@ -650,7 +656,7 @@ class ScatterChart:
                    .rename(columns={'corpus': 'cat'}))
         json_df['cat25k'] = (((json_df['cat'] * 1.
                                / json_df['cat'].sum()) * 25000)
-                             .apply(np.round).astype(np.int))
+                             .apply(np.round).astype(int))
 
         self._add_x_and_y_coords_to_term_df_if_injected(json_df)
         j = {}
