@@ -50,7 +50,6 @@ class TermDocMatrixWithoutCategories(object):
         self._strict_unigram_definition = False
         return self
 
-
     def compact(self, compactor, non_text=False):
         '''
         Compact term document matrix.
@@ -151,8 +150,8 @@ class TermDocMatrixWithoutCategories(object):
     def _get_corpus_unigram_freq(self, corpus_freq_df):
         unigram_validator = re.compile('^[A-Za-z]+$')
         corpus_unigram_freq = corpus_freq_df.loc[[term for term
-                                                 in corpus_freq_df.index
-                                                 if unigram_validator.match(term) is not None]]
+                                                  in corpus_freq_df.index
+                                                  if unigram_validator.match(term) is not None]]
         return corpus_unigram_freq
 
     def _get_background_unigram_frequencies(self):
@@ -206,14 +205,32 @@ class TermDocMatrixWithoutCategories(object):
     def _get_relevant_idx_store(self, non_text):
         return self._metadata_idx_store if non_text else self._term_idx_store
 
-    def remove_infrequent_words(self, minimum_term_count, term_ranker=AbsoluteFrequencyRanker):
+    def remove_infrequent_words(self, minimum_term_count, term_ranker=AbsoluteFrequencyRanker, non_text=False):
         '''
         Returns
         -------
         A new TermDocumentMatrix consisting of only terms which occur at least minimum_term_count.
         '''
-        tdf = term_ranker(self).get_ranks().sum(axis=1)
-        return self.remove_terms(list(tdf[tdf <= minimum_term_count].index))
+        ranker = term_ranker(self)
+        if non_text:
+            ranker = ranker.use_non_text_features()
+        tdf = ranker.get_ranks().sum(axis=1)
+
+        return self.remove_terms(list(tdf[tdf <= minimum_term_count].index), non_text=non_text)
+
+    def remove_word_by_document_pct(self, min_document_pct=0., max_document_pct=1., non_text=False):
+        '''
+        Returns a copy of the corpus with terms that occur in a document percentage range.
+
+        :param min_document_pct: float, minimum document percentage. 0 by default
+        :param max_document_pct: float, maximum document percentage. 1 by default
+        :param non_text: bool, use metadata?
+        :return: Corpus
+        '''
+        tdm = (self.get_metadata_doc_mat() > 0) if non_text else (self.get_term_doc_mat() > 0)
+        tdmpct = (tdm.sum(axis=0) / tdm.shape[0]).A1
+        mask = (tdmpct >= min_document_pct) & (tdmpct <= max_document_pct)
+        return self.remove_terms(np.array(self.get_terms())[mask])
 
     def remove_entity_tags(self):
         '''
@@ -247,7 +264,6 @@ class TermDocMatrixWithoutCategories(object):
         idx_to_delete_list = self._build_term_index_list(ignore_absences, terms, non_text)
         return self.remove_terms_by_indices(idx_to_delete_list, non_text)
 
-
     def whitelist_terms(self, whitelist_terms):
         '''
 
@@ -300,6 +316,44 @@ class TermDocMatrixWithoutCategories(object):
         terms_to_remove = np.where(term_counts < threshold)[0]
         return self.remove_terms_by_indices(terms_to_remove, non_text)
 
+    def remove_document_ids(self, document_ids, remove_unused_terms=True, remove_unused_metadata=False):
+        '''
+
+        :param document_ids: List[int], list of document ids to remove
+        :return: Corpus
+        '''
+        y_mask = ~np.isin(np.arange(self.get_num_docs()), np.array(document_ids))
+        updated_tdm = self._make_new_term_doc_matrix(
+            new_X=self._X,
+            new_mX=self._mX,
+            new_y=None,
+            new_category_idx_store=None,
+            new_term_idx_store=self._term_idx_store,
+            new_metadata_idx_store=self._metadata_idx_store,
+            new_y_mask=y_mask
+        )
+
+        if remove_unused_terms:
+            unused_term_idx = np.where(self._X[y_mask, :].sum(axis=0) == 0)[1]
+            updated_tdm = updated_tdm.remove_terms_by_indices(unused_term_idx, non_text=False)
+
+        if remove_unused_metadata:
+            unused_metadata_mask = np.mask(self._mX[y_mask, :].sum(axis=0) == 0)[0]
+            updated_tdm = updated_tdm.remove_terms_by_indices(unused_metadata_mask, non_text=True)
+
+        return updated_tdm
+
+    def remove_documents_less_than_length(self, max_length, non_text=False):
+        '''
+            `
+
+        :param max_length: int, length of document in terms registered in corpus
+        :return: Corpus
+        '''
+        tdm = self.get_metadata_doc_mat() if non_text else self.get_term_doc_mat()
+        doc_ids_to_remove = np.where(tdm.sum(axis=1).T.A1 < max_length)
+        return self.remove_document_ids(doc_ids_to_remove)
+
     def get_unigram_corpus(self):
         '''
         Returns
@@ -313,7 +367,7 @@ class TermDocMatrixWithoutCategories(object):
         return [term for term
                 in self._term_idx_store._i2val
                 if ' ' in term or (self._strict_unigram_definition and "'" in term)
-        ]
+                ]
 
     def get_stoplisted_unigram_corpus(self, stoplist=None):
         '''
@@ -347,6 +401,14 @@ class TermDocMatrixWithoutCategories(object):
         return self._remove_terms_from_list(set(self.get_default_stoplist())
                                             | set(w.lower() for w in custom_stoplist))
 
+    def filter_out(self, filter_func):
+        '''
+
+        :param filter_func: function which takes a string and returns true or false
+        :return: A new TermDocumentMatrix consisting of only unigrams in the current TermDocumentMatrix.
+        '''
+        return self.remove_terms([x for x in self.get_terms() if filter_func(x)])
+
     def _remove_terms_from_list(self, stoplist):
         terms_to_ignore = [term for term
                            in self._term_idx_store._i2val
@@ -444,7 +506,6 @@ class TermDocMatrixWithoutCategories(object):
         '''
         return self._X.astype(np.double).tocoo()
 
-
     def get_metadata_doc_mat(self):
         '''
         Returns sparse matrix representation of term-doc-matrix
