@@ -1,7 +1,11 @@
 from __future__ import print_function
 
-version = [0, 0, 2, 70]
+from scattertext.features.CognitiveDistortionsOffsetGetter import LexiconFeatAndOffsetGetter, \
+    COGNITIVE_DISTORTIONS_LEXICON, COGNITIVE_DISTORTIONS_DEFINITIONS
+
+version = [0, 1, 5]
 __version__ = '.'.join([str(e) for e in version])
+
 import re
 import numpy as np
 import pandas as pd
@@ -12,7 +16,7 @@ from scattertext.graphs.ComponentDiGraph import ComponentDiGraph
 from scattertext.graphs.ComponentDiGraphHTMLRenderer import ComponentDiGraphHTMLRenderer
 from scattertext.graphs.GraphStructure import GraphStructure
 from scattertext.graphs.SimpleDiGraph import SimpleDiGraph
-from scattertext.features.FeastFromSentencePiece import FeatsFromSentencePiece
+from scattertext.features.FeatsFromSentencePiece import FeatsFromSentencePiece
 from scattertext.features.PyTextRankPhrases import PyTextRankPhrases
 from scattertext.termscoring.BetaPosterior import BetaPosterior
 from scattertext.semioticsquare.SemioticSquareFromAxes import SemioticSquareFromAxes
@@ -26,7 +30,7 @@ import scattertext.viz
 from scattertext.CategoryColorAssigner import CategoryColorAssigner
 from scattertext.representations.EmbeddingsResolver import EmbeddingsResolver
 from scattertext.termcompaction.AssociationCompactor import AssociationCompactor, TermCategoryRanker, \
-    AssociationCompactorByRank
+    AssociationCompactorByRank, JSDCompactor
 from scattertext.termscoring.CohensD import CohensD, HedgesR
 from scattertext.termscoring.MannWhitneyU import MannWhitneyU
 from scattertext.diachronic.BubbleDiachronicVisualization import BubbleDiachronicVisualization
@@ -37,13 +41,13 @@ from scattertext.CorpusFromFeatureDict import CorpusFromFeatureDict
 from scattertext.TermCategoryFrequencies import TermCategoryFrequencies
 from scattertext.features.FeatsFromTopicModel import FeatsFromTopicModel
 from scattertext.termscoring.BM25Difference import BM25Difference
-from scattertext import SampleCorpora, SampleLexicons
+from scattertext import SampleCorpora, SampleLexicons, smoothing
 from scattertext import Scalers, ScatterChart
 from scattertext import termranking
 from scattertext.AsianNLP import chinese_nlp, japanese_nlp
 from scattertext.AutoTermSelector import AutoTermSelector
 from scattertext.CSRMatrixTools import CSRMatrixFactory
-from scattertext.Common import DEFAULT_MINIMUM_TERM_FREQUENCY, DEFAULT_PMI_THRESHOLD_COEFFICIENT
+from scattertext.Common import DEFAULT_MINIMUM_TERM_FREQUENCY, DEFAULT_PMI_THRESHOLD_COEFFICIENT, MY_ENGLISH_STOP_WORDS
 from scattertext.Corpus import Corpus
 from scattertext.CorpusFromPandas import CorpusFromPandas
 from scattertext.CorpusFromParsedDocuments import CorpusFromParsedDocuments
@@ -117,8 +121,29 @@ from scattertext.features.FeatsFromScoredLexicon import FeatsFromScoredLexicon
 from scattertext.features.SpacyEntities import SpacyEntities
 from scattertext.diachronic.TimeStructure import TimeStructure
 from scattertext.features.PyatePhrases import PyatePhrases
+from scattertext.termscoring.DeltaJSDivergence import DeltaJSDivergence
+from scattertext.CorpusFromTermFrequencies import CorpusFromTermFrequencies
+from scattertext.helpers.MakeUnique import make_unique
+from scattertext.viz.TermInfo import get_tooltip_js_function, get_custom_term_info_js_function
+from scattertext.CorpusWithoutCategoriesFromParsedDocuments import CorpusWithoutCategoriesFromParsedDocuments
+from scattertext.OffsetCorpus import OffsetCorpus
+from scattertext.OffsetCorpusFactory import OffsetCorpusFactory
+from scattertext.dispersion.Dispersion import Dispersion
+from scattertext.features import featoffsets
+from scattertext.features.featoffsets.feat_and_offset_getter import FeatAndOffsetGetter
+from scattertext.features.featoffsets.token_and_feat_offset_getter import TokenFeatAndOffsetGetter
+from scattertext.tokenizers.roberta import RobertaTokenizerWrapper
+from scattertext.continuous.sklearnpipeline import RidgeCoefficients
+from scattertext.continuous.ungar import UngarCoefficients
+from scattertext.features.RegexFeatAndOffsetGetter import RegexFeatAndOffsetGetter
+from scattertext.representations.LatentSemanticScaling import latent_semantic_scale_from_word2vec, lss_terms
+from scattertext.categorytable import CategoryTableMaker
+
+PhraseFeatsFromTopicModel = FeatsFromTopicModel  # Ensure backwards compatibility
+
+def cognitive_distortions_offset_getter_factory():
+    return LexiconFeatAndOffsetGetter(COGNITIVE_DISTORTIONS_LEXICON, COGNITIVE_DISTORTIONS_DEFINITIONS)
 
-PhraseFeatsFromTopicModel = FeatsFromTopicModel # Ensure backwards compatibility
 
 def produce_scattertext_explorer(corpus,
                                  category,
@@ -183,6 +208,8 @@ def produce_scattertext_explorer(corpus,
                                  term_scorer=None,
                                  show_axes=True,
                                  show_axes_and_cross_hairs=False,
+                                 show_diagonal=False,
+                                 use_global_scale=False,
                                  horizontal_line_y_position=None,
                                  vertical_line_x_position=None,
                                  show_cross_axes=True,
@@ -205,11 +232,39 @@ def produce_scattertext_explorer(corpus,
                                  div_name=None,
                                  alternative_term_func=None,
                                  term_metadata=None,
+                                 term_metadata_df=None,
                                  max_overlapping=-1,
                                  include_all_contexts=False,
                                  show_corpus_stats=True,
                                  sort_doc_labels_by_name=False,
+                                 enable_term_category_description=True,
                                  always_jump=True,
+                                 get_custom_term_html=None,
+                                 header_names=None,
+                                 header_sorting_algos=None,
+                                 ignore_categories=False,
+                                 d3_color_scale=None,
+                                 background_labels=None,
+                                 tooltip_columns=None,
+                                 tooltip_column_names=None,
+                                 term_description_columns=None,
+                                 term_description_column_names=None,
+                                 term_word_in_term_description='Term',
+                                 color_column=None,
+                                 color_score_column=None,
+                                 label_priority_column=None,
+                                 text_color_column=None,
+                                 suppress_text_column=None,
+                                 background_color=None,
+                                 left_list_column=None,
+                                 censor_point_column=None,
+                                 right_order_column=None,
+                                 line_coordinates=None,
+                                 subword_encoding=None,
+                                 top_terms_length=14,
+                                 top_terms_left_buffer=0,
+                                 dont_filter=False,
+                                 use_offsets=False,
                                  return_data=False,
                                  return_scatterplot_structure=False):
     '''Returns html code of visualization.
@@ -248,8 +303,8 @@ def produce_scattertext_explorer(corpus,
         Maximum number of snippets to show when term is clicked.  If None, all are shown.
     max_docs_per_category: int, optional
         Maximum number of documents to store per category.  If None, by default, all are stored.
-    metadata : list, optional
-        list of meta data strings that will be included for each document
+    metadata : list or function, optional
+        list of meta data strings that will be included for each document, if a function, called on corpus
     scores : np.array, optional
         Array of term scores or None.
     x_coords : np.array, optional
@@ -366,6 +421,11 @@ def produce_scattertext_explorer(corpus,
         Show the ticked axes on the plot.  If false, show inner axes as a crosshair.
     show_axes_and_cross_hairs : bool, default False
         Show both peripheral axis labels and cross axes.
+    show_diagonal : bool, default False
+        Show a diagonal line leading from the lower-left ot the upper-right; only makes
+        sense to use this if use_global_scale is true.
+    use_global_scale : bool, default False
+        Use same scale for both axes
     vertical_line_x_position : float, default None
     horizontal_line_y_position : float, default None
     show_cross_axes : bool, default True
@@ -417,6 +477,8 @@ def produce_scattertext_explorer(corpus,
     term_metadata : dict, None by default
         Dict mapping terms to dictionaries containing additional information which can be used in the color_func
         or the get_tooltip_content function. These will appear in termDict.etc
+    term_metadata_df : pd.DataFrame, None by default
+        Dataframe version of term_metadata
     include_all_contexts: bool, default False
         Include all contexts, even non-matching ones, in interface
     max_overlapping: int, default -1
@@ -427,6 +489,52 @@ def produce_scattertext_explorer(corpus,
         If unified, sort the document labels by name
     always_jump: bool, default True
         Always jump to term contexts if a term is clicked
+    enable_term_category_description: bool, default True
+        List term/metadata statistics under category
+    get_custom_term_html: str, default None
+        Javascript function which displays term summary from term info
+    header_names: Dict[str, str], default None
+        Dictionary giving names of term lists shown to the right of the plot. Valid keys are
+        upper, lower and right.
+    header_sorting_algos: Dict[str, str], default None
+        Dictionary giving javascript sorting algorithms for panes. Valid keys are upper, lower
+        and right. Value is a JS function which takes the "data" object.
+    ignore_categories: bool, default False
+        Signals the plot shouldn't display category names. Used in single category plots.
+    suppress_text_column: str, default None
+        Column in term_metadata_df which indicates term should be hidden
+    left_list_column: str, default None
+        Column in term_metadata_df which should be used for sorting words into upper and lower
+        parts of left word-list sections. Highest values in upper, lowest in lower.
+    tooltip_columns: List[str]
+    tooltip_column_names: Dict[str, str]
+    term_description_columns: List[str]
+    term_description_column_names: Dict[str]
+    term_word_in_term_description: str, default None
+    color_column: str, default None:
+        column in term_metadata_df which indicates color
+    color_score_column: str, default None
+        column in term_metadata df; contains value between 0 and 1 which will be used to assign a color
+    label_priority_column : str, default None
+        Column in term_metadata_df; larger values in the column indicate a term should be labeled first
+    censor_point_column : str, default None
+        Should we allow labels to be drawn over point?
+    right_order_column : str, default None
+        Order for right column ("characteristic" by default); largest first
+    background_color : str, default None
+        Changes document.body's background color to background_color
+    line_coordinates : list, default None
+        Coordinates for drawing a line under the plot
+    subword_encoding : str, default None
+        Type of subword encoding to use, None if none, currently supports "RoBERTa"
+    top_terms_length : int, default 14
+        Number of words to list in most/least associated lists on left-hand side
+    top_terms_left_buffer : int, default 0
+        Number of pixels left to shift top terms list
+    dont_filter : bool, default False
+        Don't filter any terms when charting
+    use_offsets : bool, default False
+        Enable the use of metadata offsets
     return_data : bool default False
         Return a dict containing the output of `ScatterChartExplorer.to_dict` instead of
         an html.
@@ -438,9 +546,8 @@ def produce_scattertext_explorer(corpus,
     html of visualization
 
     '''
-    color = None
     if singleScoreMode or word_vec_use_p_vals:
-        color = 'd3.interpolatePurples'
+        d3_color_scale = 'd3.interpolatePurples'
     if singleScoreMode or not sort_by_dist:
         sort_by_dist = False
     else:
@@ -476,7 +583,8 @@ def produce_scattertext_explorer(corpus,
                                                   term_ranker=term_ranker,
                                                   use_non_text_features=use_non_text_features,
                                                   term_significance=term_significance,
-                                                  terms_to_include=terms_to_include, )
+                                                  terms_to_include=terms_to_include,
+                                                  dont_filter=dont_filter,)
     if ((x_coords is None and y_coords is not None)
             or (y_coords is None and x_coords is not None)):
         raise Exception("Both x_coords and y_coords need to be passed or both left blank")
@@ -493,6 +601,11 @@ def produce_scattertext_explorer(corpus,
         scatter_chart_explorer.inject_metadata_descriptions(metadata_descriptions)
     if term_colors is not None:
         scatter_chart_explorer.inject_term_colors(term_colors)
+
+    if term_metadata_df is not None and term_metadata is not None:
+        raise Exception("Both term_metadata_df and term_metadata cannot be values which are not None.")
+    if term_metadata_df is not None:
+        scatter_chart_explorer.inject_term_metadata_df(term_metadata_df)
     if term_metadata is not None:
         scatter_chart_explorer.inject_term_metadata(term_metadata)
     html_base = None
@@ -507,24 +620,70 @@ def produce_scattertext_explorer(corpus,
         transform=transform,
         scores=scores,
         max_docs_per_category=max_docs_per_category,
-        metadata=metadata,
+        metadata=metadata if not callable(metadata) else metadata(corpus),
         alternative_text_field=alternative_text_field,
         neutral_category_name=neutral_category_name,
         extra_category_name=extra_category_name,
         neutral_categories=neutral_categories,
         extra_categories=extra_categories,
         background_scorer=characteristic_scorer,
-        include_term_category_counts=include_term_category_counts
+        include_term_category_counts=include_term_category_counts,
+        use_offsets=use_offsets
     )
 
+    if line_coordinates is not None:
+        scatter_chart_data['line'] = line_coordinates
+
     if return_data:
         return scatter_chart_data
 
+    if tooltip_columns is not None:
+        assert get_tooltip_content is None
+        get_tooltip_content = get_tooltip_js_function(
+            term_metadata_df,
+            tooltip_column_names,
+            tooltip_columns
+        )
+
+    if term_description_columns is not None:
+        assert get_custom_term_html is None
+        get_custom_term_html = get_custom_term_info_js_function(
+            term_metadata_df,
+            term_description_column_names,
+            term_description_columns,
+            term_word_in_term_description
+        )
+
+    if color_column:
+        assert color_func is None
+        color_func = '(function(d) {return d.etc["%s"]})' % color_column
+
+    if color_score_column:
+        assert color_func is None
+        color_func = '(function(d) {return %s(d.etc["%s"])})' % (
+            d3_color_scale if d3_color_scale is not None else 'd3.interpolateRdYlBu',
+            color_score_column
+        )
+
+    if header_sorting_algos is not None:
+        assert 'upper' not in header_sorting_algos
+        assert 'lower' not in header_sorting_algos
+    if left_list_column is not None:
+        assert term_metadata_df is not None
+        assert left_list_column in term_metadata_df
+        header_sorting_algos = {
+            "upper": '((a,b) => b.etc["' + left_list_column + '"] - a.etc["' + left_list_column + '"])',
+            "lower": '((a,b) => a.etc["' + left_list_column + '"] - b.etc["' + left_list_column + '"])'
+        }
+    if right_order_column is not None:
+        assert right_order_column in term_metadata_df
+
     scatterplot_structure = ScatterplotStructure(VizDataAdapter(scatter_chart_data),
                                                  width_in_pixels=width_in_pixels,
                                                  height_in_pixels=height_in_pixels,
                                                  max_snippets=max_snippets,
-                                                 color=color, grey_zero_scores=gray_zero_scores,
+                                                 color=d3_color_scale,
+                                                 grey_zero_scores=gray_zero_scores,
                                                  sort_by_dist=sort_by_dist,
                                                  reverse_sort_scores_for_not_category=reverse_sort_scores_for_not_category,
                                                  use_full_doc=use_full_doc,
@@ -562,12 +721,29 @@ def produce_scattertext_explorer(corpus,
                                                  alternative_term_func=alternative_term_func,
                                                  include_all_contexts=include_all_contexts,
                                                  show_axes_and_cross_hairs=show_axes_and_cross_hairs,
+                                                 show_diagonal=show_diagonal,
+                                                 use_global_scale=use_global_scale,
                                                  x_axis_values_format=x_axis_values_format,
                                                  y_axis_values_format=y_axis_values_format,
                                                  max_overlapping=max_overlapping,
                                                  show_corpus_stats=show_corpus_stats,
                                                  sort_doc_labels_by_name=sort_doc_labels_by_name,
-                                                 always_jump=always_jump)
+                                                 enable_term_category_description=enable_term_category_description,
+                                                 always_jump=always_jump,
+                                                 get_custom_term_html=get_custom_term_html,
+                                                 header_names=header_names,
+                                                 header_sorting_algos=header_sorting_algos,
+                                                 ignore_categories=ignore_categories,
+                                                 background_labels=background_labels,
+                                                 label_priority_column=label_priority_column,
+                                                 text_color_column=text_color_column,
+                                                 suppress_text_column=suppress_text_column,
+                                                 background_color=background_color,
+                                                 censor_point_column=censor_point_column,
+                                                 right_order_column=right_order_column,
+                                                 subword_encoding=subword_encoding,
+                                                 top_terms_length=top_terms_length,
+                                                 top_terms_left_buffer=top_terms_left_buffer)
 
     if return_scatterplot_structure:
         return scatterplot_structure
@@ -780,7 +956,7 @@ def word_similarity_explorer(corpus,
     target_term : str
         Word or phrase for semantic similarity comparison
     nlp : spaCy-like parsing function
-        E.g., spacy.load('en'), whitespace_nlp, etc...
+        E.g., spacy.load('en_core_web_sm'), whitespace_nlp, etc...
     alpha : float, default = 0.01
         Uniform dirichlet prior for p-value calculation
     max_p_val : float, default = 0.1
@@ -793,7 +969,7 @@ def word_similarity_explorer(corpus,
 
     if nlp is None:
         import spacy
-        nlp = spacy.load('en')
+        nlp = spacy.load('en_core_web_sm')
 
     base_term = nlp(target_term)
     scores = np.array([base_term.similarity(nlp(tok))
@@ -1664,8 +1840,8 @@ def produce_scattertext_digraph(
         metadata_func=None,
         enable_pan_and_zoom=True,
         engine='dot',
-        graph_params = None,
-        node_params = None,
+        graph_params=None,
+        node_params=None,
         **kwargs
 ):
     '''
@@ -1756,3 +1932,157 @@ def produce_scattertext_digraph(
     return html
 
 
+def dataframe_scattertext(
+        corpus,
+        plot_df,
+        **kwargs
+):
+    assert 'X' in plot_df
+    assert 'Y' in plot_df
+    if 'Xpos' not in plot_df:
+        plot_df['Xpos'] = Scalers.scale(plot_df['X'])
+    if 'Ypos' not in plot_df:
+        plot_df['Ypos'] = Scalers.scale(plot_df['Y'])
+
+    assert len(plot_df) > 0
+
+    if 'term_description_columns' not in kwargs:
+        kwargs['term_description_columns'] = [x for x in plot_df.columns if x not in
+                                              ['X', 'Y', 'Xpos', 'Ypos']]
+
+    if 'tooltip_columns' not in kwargs:
+        kwargs['tooltip_columns'] = ['Xpos', 'Ypos']
+        kwargs['tooltip_column_names'] = {'Xpos': kwargs.get('x_label', 'X'), 'Ypos': kwargs.get('y_label', 'Y')}
+
+    kwargs.setdefault('metadata', None),
+    kwargs.setdefault('scores', plot_df['Score'] if 'Score' in plot_df else 0),
+    kwargs.setdefault('minimum_term_frequency', 0)
+    kwargs.setdefault('pmi_threshold_coefficient', 0)
+    kwargs.setdefault('category', corpus.get_categories()[0])
+    kwargs.setdefault('original_x', plot_df['X'].values)
+    kwargs.setdefault('original_y', plot_df['Y'].values)
+    kwargs.setdefault('x_coords', plot_df['Xpos'].values)
+    kwargs.setdefault('y_coords', plot_df['Ypos'].values)
+    kwargs.setdefault('use_global_scale', True)
+    kwargs.setdefault('ignore_categories', True)
+    kwargs.setdefault('unified_context', kwargs['ignore_categories'])
+    kwargs.setdefault('show_axes_and_cross_hairs', 0)
+    kwargs.setdefault('show_top_terms', False)
+    kwargs.setdefault('x_label', 'X')
+    kwargs.setdefault('y_label', 'Y')
+    return produce_scattertext_explorer(
+        corpus,
+        term_metadata_df=plot_df,
+        **kwargs
+    )
+
+
+class TableStructure(GraphStructure):
+    def __init__(self,
+                 scatterplot_structure,
+                 graph_renderer,
+                 **kwargs):
+        kwargs.setdefault('template_file_name', 'table_plot.html')
+        GraphStructure.__init__(self, scatterplot_structure, graph_renderer, **kwargs)
+
+    def get_font_import(self):
+        return ''
+
+    def get_zoom_script_import(self):
+        return ''
+
+
+def produce_scattertext_table(
+        corpus,
+        num_rows=10,
+        use_non_text_features=False,
+        plot_width=500,
+        plot_height=700,
+        category_order=None,
+        **kwargs
+):
+    '''
+
+    :param df: pd.DataFrame
+    :param text_col: str
+    :param source_col: str
+    :param dest_col: str
+    :param source_name: str
+    :param dest_name: str
+    :param plot_width: int
+    :param plot_height: int
+    :param enable_pan_and_zoom: bool
+    :param engine: str, The graphviz engine (e.g., dot or neat)
+    :param graph_params dict or None, graph parameters in graph viz
+    :param node_params dict or None, node parameters in graph viz
+    :param category_order list or None, names of categories to show in order
+    :param kwargs: dict
+    :return: str
+    '''
+
+    alternative_term_func = '''(function(termDict) {
+       //document.querySelectorAll(".dotgraph").forEach(svg => svg.style.display = 'none');
+       //showTermGraph(termDict['term']);
+       //alert(termDict['term'])
+       return true;
+    })'''
+
+    graph_renderer = CategoryTableMaker(
+        corpus=corpus,
+        num_rows=num_rows,
+        use_metadata=use_non_text_features,
+        category_order=category_order
+    )
+
+    dispersion = Dispersion(
+        corpus, use_categories=True, use_metadata=use_non_text_features
+    )
+
+    adjusted_dispersion = dispersion.get_adjusted_metric(
+        dispersion.da(),
+        dispersion.get_frequency()
+    )
+
+    plot_df = pd.DataFrame().assign(
+        X=dispersion.get_frequency(),
+        Frequency=lambda df: df.X,
+        Xpos=lambda df: Scalers.dense_rank(df.X),
+        Y=lambda df: adjusted_dispersion,
+        AdjustedDA=lambda df: df.Y,
+        Ypos=lambda df: Scalers.scale_neg_1_to_1_with_zero_mean(df.Y),
+        ColorScore=lambda df: Scalers.scale_neg_1_to_1_with_zero_mean(df.Y),
+        term=dispersion.get_names()
+    ).set_index('term')
+
+    line_df = pd.DataFrame({
+        'x': plot_df.Xpos.values,
+        'y': 0.5,
+    }).sort_values(by='x')
+    kwargs.setdefault('top_terms_left_buffer', 10)
+    scatterplot_structure = dataframe_scattertext(
+        corpus,
+        plot_df=plot_df,
+        ignore_categories=False,
+        unified_context=kwargs.get('unified_context', True),
+        x_label='Frequency Rank',
+        y_label='Frequency-adjusted DA',
+        y_axis_labels=['More Concentrated', 'Medium', 'More Dispersion'],
+        color_score_column='ColorScore',
+        tooltip_columns=['Frequency', 'AdjustedDA'],
+        header_names={'upper': 'Dispersed', 'lower': 'Concentrated'},
+        left_list_column='AdjustedDA',
+        line_coordinates=line_df.to_dict('records'),
+        use_non_text_features=use_non_text_features,
+        return_scatterplot_structure=True,
+        width_in_pixels=plot_width,
+        height_in_pixels=plot_height,
+        #alternative_term_func=alternative_term_func,
+        **kwargs
+    )
+
+    html = TableStructure(
+        scatterplot_structure,
+        graph_renderer=graph_renderer
+    ).to_html()
+
+    return html
