@@ -6,6 +6,6 @@ class FeatsFromOnlyEmpath(FeatsFromSpacyDocAndEmpath):
 	def get_feats(self, doc):
 		return Counter()
 	def get_doc_metadata(self, doc, prefix=''):
-		return super(FeatsFromOnlyEmpath, self).get_doc_metadata(doc, prefix=prefix)
+		return FeatsFromSpacyDocAndEmpath.get_doc_metadata(self, doc, prefix=prefix)
 
 
