@@ -19,10 +19,12 @@ class TestHTMLVisualizationAssembly(TestCase):
                   'null', 'false', 'false',
                   '"' + DEFAULT_D3_AXIS_VALUE_FORMAT + '"',
                   '"' + DEFAULT_D3_AXIS_VALUE_FORMAT + '"',
-                  'false', '-1', 'true', 'false', 'true', 'false']
+                  'false', '-1', 'true', 'false', 'true', 'false', 'false', 'false', 'true', 'null', 'null', 'null',
+                  'false', 'null', 'undefined', 'undefined', 'undefined', 'undefined', 'undefined', 'undefined',
+                  'undefined', '14', '0']
         for i, val in param_dict.items():
             params[i] = val
-        return 'buildViz(' + ','.join(params) + ');'
+        return 'buildViz(' + ',\n'.join(params) + ');\n'
 
     def make_assembler(self):
         scatterplot_structure = ScatterplotStructure(self.make_adapter())
@@ -262,7 +264,7 @@ class TestHTMLVisualizationAssembly(TestCase):
                           .call_build_visualization_in_javascript()),
                          self.get_params({21: "[1, 2, 3]"}))
 
-    def test_x_axis_labels(self):
+    def test_y_axis_labels(self):
         visualization_data = self.make_adapter()
         self.assertEqual((ScatterplotStructure(visualization_data, y_axis_values=[4, 5, 6])
                           .call_build_visualization_in_javascript()),
@@ -402,7 +404,6 @@ class TestHTMLVisualizationAssembly(TestCase):
                   .call_build_visualization_in_javascript())
         self.assertEqual(params, self.get_params({44: '10'}))
 
-
     def test_show_corpus_stats(self):
         visualization_data = self.make_adapter()
         params = (ScatterplotStructure(visualization_data, show_corpus_stats=False)
@@ -428,3 +429,127 @@ class TestHTMLVisualizationAssembly(TestCase):
                                        highlight_selected_category=True)
                   .call_build_visualization_in_javascript())
         self.assertEqual(params, self.get_params({48: 'true'}))
+
+    def test_show_diagonal(self):
+        visualization_data = self.make_adapter()
+        params = (ScatterplotStructure(visualization_data,
+                                       show_diagonal=True)
+                  .call_build_visualization_in_javascript())
+        self.assertEqual(params, self.get_params({49: 'true'}))
+
+    def test_use_global_scale(self):
+        visualization_data = self.make_adapter()
+        params = (ScatterplotStructure(visualization_data, use_global_scale=True)
+                  .call_build_visualization_in_javascript())
+        self.assertEqual(params, self.get_params({50: 'true'}))
+
+    def test_enable_term_category_description(self):
+        visualization_data = self.make_adapter()
+        params = (ScatterplotStructure(visualization_data, enable_term_category_description=False)
+                  .call_build_visualization_in_javascript())
+        self.assertEqual(params, self.get_params({51: 'false'}))
+
+    def test_get_custom_term_html(self):
+        visualization_data = self.make_adapter()
+        html = '(function(x) {return "Term: " + x.term})'
+        params = (ScatterplotStructure(
+            visualization_data,
+            get_custom_term_html=html
+        ).call_build_visualization_in_javascript())
+        self.assertEqual(params, self.get_params({52: html}))
+
+    def test_header_names(self):
+        visualization_data = self.make_adapter()
+        header_names = {'upper': 'Upper Header Name', 'lower': 'Lower Header Name'}
+        params = (ScatterplotStructure(
+            visualization_data,
+            header_names=header_names
+        ).call_build_visualization_in_javascript())
+        self.assertEqual(params, self.get_params(
+            {53: '''{"upper": "Upper Header Name", "lower": "Lower Header Name"}'''}
+        ))
+
+    def test_header_sorting_algos(self):
+        visualization_data = self.make_adapter()
+        header_sorting_algos = {'upper': '(function(a, b) {return b.s - a.s})',
+                                'lower': '(function(a, b) {return a.s - b.s})'}
+        params = (ScatterplotStructure(
+            visualization_data,
+            header_sorting_algos=header_sorting_algos
+        ).call_build_visualization_in_javascript())
+        self.assertEqual(params, self.get_params(
+            {54: '''{"lower": (function(a, b) {return a.s - b.s}), "upper": (function(a, b) {return b.s - a.s})}'''}
+        ))
+
+    def test_ignore_categories(self):
+        visualization_data = self.make_adapter()
+        params = (ScatterplotStructure(visualization_data, ignore_categories=True)
+                  .call_build_visualization_in_javascript())
+        self.assertEqual(params, self.get_params({55: 'true'}))
+
+    def test_background_labels(self):
+        visualization_data = self.make_adapter()
+        params = (ScatterplotStructure(visualization_data, background_labels=[
+            {'Text': 'Topic 0', 'X': 0.5242579971278757, 'Y': 0.8272937510221724},
+            {'Text': 'Topic 1', 'X': 0.7107755717675702, 'Y': 0.5034326824672314},
+            {'Text': 'Topic 2', 'X': 0.09014690078982, 'Y': 0.6261596586530888}])
+                  .call_build_visualization_in_javascript())
+        self.assertEqual(params, self.get_params(
+            {56: '[{"Text": "Topic 0", "X": 0.5242579971278757, "Y": 0.8272937510221724}, '
+                 '{"Text": "Topic 1", "X": 0.7107755717675702, "Y": 0.5034326824672314}, '
+                 '{"Text": "Topic 2", "X": 0.09014690078982, "Y": 0.6261596586530888}]'}))
+
+    def test_label_priority_column(self):
+        visualization_data = self.make_adapter()
+        params = (ScatterplotStructure(visualization_data, label_priority_column='LabelPriority')
+                  .call_build_visualization_in_javascript())
+        self.assertEqual(params, self.get_params({57: '"LabelPriority"'}))
+
+    def test_text_color_column(self):
+        visualization_data = self.make_adapter()
+        params = (ScatterplotStructure(visualization_data, text_color_column='TextColor')
+                  .call_build_visualization_in_javascript())
+        self.assertEqual(params, self.get_params({58: '"TextColor"'}))
+
+    def test_suppress_label_column(self):
+        visualization_data = self.make_adapter()
+        params = (ScatterplotStructure(visualization_data, suppress_text_column='Suppress')
+                  .call_build_visualization_in_javascript())
+        self.assertEqual(params, self.get_params({59: '"Suppress"'}))
+
+    def test_background_color(self):
+        visualization_data = self.make_adapter()
+        params = (ScatterplotStructure(visualization_data, background_color='#444444')
+                  .call_build_visualization_in_javascript())
+        self.assertEqual(params, self.get_params({60: '"#444444"'}))
+
+    def test_censor_point_column(self):
+        visualization_data = self.make_adapter()
+        params = (ScatterplotStructure(visualization_data, censor_point_column='CensorPoint')
+                  .call_build_visualization_in_javascript())
+        self.assertEqual(params, self.get_params({61: '"CensorPoint"'}))
+
+    def test_right_order_column(self):
+        visualization_data = self.make_adapter()
+        params = (ScatterplotStructure(visualization_data, right_order_column='Priority')
+                  .call_build_visualization_in_javascript())
+        self.assertEqual(params, self.get_params({62: '"Priority"'}))
+
+    def test_sentence_piece(self):
+        visualization_data = self.make_adapter()
+        params = (ScatterplotStructure(visualization_data, subword_encoding='RoBERTa')
+                  .call_build_visualization_in_javascript())
+        self.assertEqual(params, self.get_params({63: '"RoBERTa"'}))
+
+
+    def test_top_terms_length(self):
+        visualization_data = self.make_adapter()
+        params = (ScatterplotStructure(visualization_data, top_terms_length=5)
+                  .call_build_visualization_in_javascript())
+        self.assertEqual(params, self.get_params({64: '5'}))
+
+    def test_top_terms_left_buffer(self):
+        visualization_data = self.make_adapter()
+        params = (ScatterplotStructure(visualization_data, top_terms_left_buffer=10)
+                  .call_build_visualization_in_javascript())
+        self.assertEqual(params, self.get_params({65: '10'}))
\ No newline at end of file
