@@ -58,6 +58,8 @@ class TestTermDocMat(TestCase):
                          [2, 2, 1])
 
     def test_single_category_term_doc_matrix_should_error(self):
+        # I'm going to let corpora be multi-category and see how it goes
+        '''
         with self.assertRaisesRegex(
                 expected_exception=CannotCreateATermDocMatrixWithASignleCategoryException,
                 expected_regex='Documents must be labeled with more than one category. '
@@ -67,6 +69,12 @@ class TestTermDocMat(TestCase):
                  for category, text
                  in get_test_categories_and_documents()]
             )
+        '''
+        single_category_tdm = build_from_category_whitespace_delimited_text(
+            [['a', text]
+             for category, text
+             in get_test_categories_and_documents()]
+        )
 
     def test_total_unigram_count(self):
         self.assertEqual(self.tdm.get_total_unigram_count(), 36)
@@ -449,6 +457,17 @@ class TestTermDocMat(TestCase):
                             == (meta_hamlet.get_category_names_by_row() == 'jay-z/r. kelly')))
         np.testing.assert_array_equal(meta_hamlet.get_term_doc_mat().todense(), hamlet.get_metadata_doc_mat().toarray())
 
+    def test_copy_terms_to_metadata(self):
+        tdm = get_hamlet_term_doc_matrix()
+        tdm_meta = tdm.copy_terms_to_metadata()
+        self.assertEqual(tdm.get_metadata_doc_mat().shape, (1,1))
+        self.assertEqual(tdm.get_term_doc_mat().shape, (211, 26875))
+        self.assertEqual(tdm_meta.get_term_doc_mat().shape, (211, 26875))
+        self.assertEqual(tdm_meta.get_metadata_doc_mat().shape, (211, 26875))
+        np.testing.assert_array_equal(tdm.get_term_freq_df().index, tdm_meta.get_metadata_freq_df().index)
+
+
+
     def test_get_num_metadata(self):
         self.assertEqual(get_hamlet_term_doc_matrix().use_categories_as_metadata().get_num_metadata(), 2)
 
