@@ -59,7 +59,25 @@ class ScatterplotStructure(object):
             max_overlapping=-1,
             show_corpus_stats=True,
             sort_doc_labels_by_name=False,
-            always_jump=True):
+            always_jump=True,
+            show_diagonal=False,
+            enable_term_category_description=True,
+            use_global_scale=False,
+            get_custom_term_html=None,
+            header_names=None,
+            header_sorting_algos=None,
+            ignore_categories=False,
+            background_labels=None,
+            label_priority_column=None,
+            text_color_column=None,
+            suppress_text_column=None,
+            censor_point_column=None,
+            background_color=None,
+            right_order_column=None,
+            subword_encoding=None,
+            top_terms_length=14,
+            top_terms_left_buffer=0
+    ):
         '''
 
         Parameters
@@ -175,7 +193,45 @@ class ScatterplotStructure(object):
             If unified document labels, sort the labels by name instead of value.
         always_jump: bool, default True
             Always jump to term contexts if a term is clicked.
-
+        show_diagonal: bool, default False
+            Show a diagonal line from the lower-left to the upper-right of the chart
+        use_global_scale: bool, default False
+            Use the same scaling for the x and y axis. Without this show_diagonal may not
+            make sense.
+        enable_term_category_description: bool, default True
+            List term/metadata statistics under category
+        get_custom_term_html: str, default None
+            Javascript function which takes term info and outputs custom term HTML
+        header_names: Dict[str, str], default None
+            Dictionary giving names of term lists shown to the right of the plot. Valid keys are
+            upper, lower and right.
+        header_sorting_algos: Dict[str, str], default None
+            Dictionary giving javascript sorting algorithms for panes. Valid keys are upper, lower
+            and right. Value is a JS function which takes the "data" object.
+        ignore_categories: bool, default False
+            Signals plot should ignore category names.
+        background_labels: List[Dict]: default None
+            List of [{"Text": "Label", "X": xpos, "Y": ypos}, ...] to be background labels on plot
+        label_priority_column : str, default None
+            Column in term_metadata_df; larger values in the column indicate a term should be labeled first
+        text_color_column: str, default None
+            Column in term_metadata_df which supplies term colors
+        suppress_text_column: str, default None
+            Column in term_metadata_df which is of boolean value. Indicates if a term should be labeled.
+        censor_point_column : str, default None
+            Should we prevent labels from being drawn over a point?
+        right_order_column : str, default None
+            Order for right column ("characteristic" by default); largest first
+        subword_encoding : str, default None
+            Type of subword encoding to use, None if none, currently supports "RoBERTa"
+        background_color: str, default None
+            Color to set document.body's background
+        sentence_piece: bool, default False
+            Use sentence piece conventions from to search for terms in JS
+        top_terms_length: int, default 14
+            Number of top terms to show for left-hand side lists
+        top_terms_lefft_buffer: int, default 0
+            Number of pixels left to shift top terms list
         '''
         self._visualization_data = visualization_data
         self._width_in_pixels = width_in_pixels if width_in_pixels is not None else 1000
@@ -227,10 +283,27 @@ class ScatterplotStructure(object):
         self._show_corpus_stats = show_corpus_stats
         self._sort_doc_labels_by_name = sort_doc_labels_by_name
         self._always_jump = always_jump
+        self._show_diagonal = show_diagonal
+        self._use_global_scale = use_global_scale
+        self._enable_term_category_description = enable_term_category_description
+        self._get_custom_term_html = get_custom_term_html
+        self._header_names = header_names
+        self._header_sorting_algos = header_sorting_algos
+        self._ignore_categories = ignore_categories
+        self._background_labels = background_labels
+        self._label_priority_column = label_priority_column
+        self._text_color_column = text_color_column
+        self._suppress_label_column = suppress_text_column
+        self._censor_point_column = censor_point_column
+        self._right_order_column = right_order_column
+        self._background_color = background_color
+        self._subword_encoding = subword_encoding
+        self._top_terms_length = top_terms_length
+        self._top_terms_left_buffer = top_terms_left_buffer
 
     def call_build_visualization_in_javascript(self):
-        def js_default_value(x):
-            return 'undefined' if x is None else str(x)
+        def js_default_value(x, default='undefined'):
+            return default if x is None else str(x)
 
         def js_default_string(x, default_string=None):
             if x is not None:
@@ -242,7 +315,7 @@ class ScatterplotStructure(object):
         def js_default_value_to_null(x):
             return 'null' if x is None else str(x)
 
-        def js_list_or_null(x):
+        def json_or_null(x):
             return 'null' if x is None else json.dumps(x)
 
         def js_bool(x):
@@ -257,6 +330,18 @@ class ScatterplotStructure(object):
         def js_default_full_data(full_data):
             return full_data if full_data is not None else "getDataAndInfo()"
 
+        def json_with_jsvalue_or_null(x):
+            if x is None:
+                return 'null'
+            to_ret = '{'
+            first = True
+            for key, val in sorted(x.items()):
+                if not first: to_ret += ', '
+                to_ret += '"%s": %s' % (key, val)
+                first = False
+            to_ret += '}'
+            return to_ret
+
         arguments = [
             js_default_value(self._width_in_pixels),
             js_default_value(self._height_in_pixels),
@@ -286,10 +371,10 @@ class ScatterplotStructure(object):
             js_bool(self._show_extra),
             js_bool(self._do_censor_points),
             js_bool(self._center_label_over_points),
-            js_list_or_null(self._x_axis_labels),
-            js_list_or_null(self._y_axis_labels),
+            json_or_null(self._x_axis_labels),
+            json_or_null(self._y_axis_labels),
             js_default_value(self._topic_model_preview_size),
-            js_list_or_null(self._vertical_lines),
+            json_or_null(self._vertical_lines),
             js_default_value_to_null(self._horizontal_line_y_position),
             js_default_value_to_null(self._vertical_line_x_position),
             js_bool(self._unified_context),
@@ -306,9 +391,26 @@ class ScatterplotStructure(object):
             js_bool(self._show_corpus_stats),
             js_bool(self._sort_doc_labels_by_name),
             js_bool(self._always_jump),
-            js_bool(self._highlight_selected_category)
+            js_bool(self._highlight_selected_category),
+            js_bool(self._show_diagonal),
+            js_bool(self._use_global_scale),
+            js_bool(self._enable_term_category_description),
+            js_default_value_to_null(self._get_custom_term_html),
+            json_or_null(self._header_names),
+            json_with_jsvalue_or_null(self._header_sorting_algos),
+            js_bool(self._ignore_categories),
+            json_or_null(self._background_labels),
+            js_default_string(self._label_priority_column),
+            js_default_string(self._text_color_column),
+            js_default_string(self._suppress_label_column),
+            js_default_string(self._background_color),
+            js_default_string(self._censor_point_column),
+            js_default_string(self._right_order_column),
+            js_default_string(self._subword_encoding),
+            js_default_value(self._top_terms_length, '14'),
+            js_default_value(self._top_terms_left_buffer, '0')
         ]
-        return 'buildViz(' + ','.join(arguments) + ');'
+        return 'buildViz(' + ',\n'.join(arguments) + ');\n'
 
     def get_js_to_call_build_scatterplot(self, object_name='plotInterface'):
         return object_name + ' = ' + self.call_build_visualization_in_javascript()
